# -*- coding:utf-8 -*-
from IPy import IP
from os import unlink, mkdir, system, sep, remove, chmod
from os.path import isfile, isdir, basename, dirname
from shutil import rmtree, copy

from arv.lib.util import trace, bin_encoding
from arv.lib.logger import logger
from arv.db import initialize as arvdb
from arv.db.edge import *
from arv.db.node import *

from arv.db.strongswan import (sw_create_identity, sw_create_certificate,
        sw_create_private_key, sw_join_certificate_identity,
        sw_join_privatekey_identity, sw_create_peer_configs,
        sw_create_child_config, sw_join_child_peer, sw_create_traffic_selector,
        sw_join_config_traffic, sw_open, sw_get_identity, sw_get_traffic_selector,
        sw_create_proposals, sw_create_ike_config_proposal, sw_create_child_config_proposal,
        sw_create_certificate_authorities, sw_create_certificate_distribution_points, sw_create_ike_configs)

from arv.config import (strongswan_database, strongswan_databases_directory,
                       proposals, ike_cfg_proposal, child_cfg_proposal, test_rvp)

def _make_cred_id(cred, typ):
    return int(str(cred.id) + typ)

@trace
def add_certificates_and_identities(nodea, creda, nodeb, credb):
        #Add cred for current in database
        #creds_a = nodea.get_credentials()
        ##test if node already in database
        #if creds_a == []:
        #    logger.debug('No creds for current node: {0}'.format(nodea.name))
        #    return
        ##Add Node in database
        #creds_b = nodeb.get_credentials()
        ##test if node already in database
        #if creds_b == []:
        #    logger.debug('No creds for remote node: {0}'.format(nodeb.name))
        #    return
        #add cred/identities only for current_node
        logger.debug("# iterate over creds for current node: {0}".format(nodea.name))
        type_nine = _make_cred_id(creda, '9')
        type_eleven_key_id = _make_cred_id(creda, '111')
        type_eleven_subjkey = _make_cred_id(creda, '112')
        if sw_get_identity(identityid=type_nine) == None:
            sw_create_identity(identityid=type_nine, data=creda.subject, nodetype=9)
            sw_create_identity(identityid=type_eleven_key_id, data=creda.keyid_cred, nodetype=11)
            sw_create_identity(identityid=type_eleven_subjkey, data=creda.subjkey, nodetype=11)

            sw_create_certificate(certid=creda.id, data=bin_encoding(creda.credential))
            sw_join_certificate_identity(identityid=type_eleven_key_id, certid=creda.id)
            sw_join_certificate_identity(identityid=type_eleven_subjkey, certid=creda.id)
            sw_join_certificate_identity(identityid=type_nine, certid=creda.id)
            sw_create_private_key(certid=creda.id, data=bin_encoding(creda.private_key))
            sw_join_privatekey_identity(keyid=creda.id, identityid=type_nine)
            sw_join_privatekey_identity(keyid=creda.id, identityid=type_eleven_key_id)
            sw_join_privatekey_identity(keyid=creda.id, identityid=type_eleven_subjkey)
        else:
            logger.debug('no certificates found for current node {0}'.format(nodea.name))

        #add cred/identities only for other node
        logger.debug("# iterate over creds for remote node: {0}".format(nodeb.name))
        type_nine = _make_cred_id(credb, '9')
        if sw_get_identity(identityid=type_nine) == None:
            sw_create_identity(identityid=type_nine, data=credb.subject, nodetype=9)
        else:
            logger.debug('no certificates found for remote node: {0}'.format(nodeb.name))

@trace
def db_apply(zephir):
    arvdb.initialize_database()
    @trace
    def create_database(current_node, database):
        @trace
        def create_traffic_selector (vertex):
            if sw_get_traffic_selector(vertex.id) == None:
                if vertex.tmpl_vertex.mimetype == u'ip':
                    start_ip = vertex.ip1
                    end_ip = vertex.ip1
                elif vertex.tmpl_vertex.mimetype == u'range':
                    start_ip = vertex.ip1
                    end_ip = vertex.ip2
                elif vertex.tmpl_vertex.mimetype == u'network':
                    ip = IP('%s/%s'%(vertex.ip1, vertex.ip2))
                    last=len(ip)-1
                    start_ip = str(ip[0])
                    end_ip = str(ip[last])
                else:
                    raise Exception('vertex has unknown mimetype %s'%vertex.mimetype)
                sw_create_traffic_selector(trafficid=vertex.id, start_addr=bin_encoding(start_ip), end_addr=bin_encoding(end_ip))

        logger.debug('current_node : {0}'.format(current_node.name))

        sw_open(database)

        # create tables from default values
        for propid, prop in proposals.items():
            sw_create_proposals(propid=propid, proposal=prop)
        for ike_cfg_prop, prop_priorities in ike_cfg_proposal.items():
            for prop in prop_priorities:
                sw_create_ike_config_proposal(ike_cfg_prop, prop[0], prop[1])

        #Add CA in database
        for cred in get_credentials_auth():
            logger.debug("# iterate over get_credentials_auth()")
            type_nine = _make_cred_id(cred, '9')
            type_eleven_key_id = _make_cred_id(cred, '111')
            sw_create_identity(type_eleven_key_id, cred.keyid_cred, nodetype=11)
            sw_create_identity(type_nine, cred.subject, nodetype=9)
            sw_create_certificate(certid=cred.id, data=bin_encoding(cred.credential))
            sw_create_certificate_authorities(authid=type_nine, certificate=cred.id)
            #add crls for the CA
            crls = Crl.query.filter_by(ca=cred).all()
            logger.debug("Crls {0}".format(crls))
            for crl in crls:
                sw_create_certificate_distribution_points(ca=type_nine, distribtype=1, uri=crl.uri)

            sw_join_certificate_identity(identityid=type_eleven_key_id, certid=cred.id)
            sw_join_certificate_identity(identityid=type_nine, certid=cred.id)
        for connect in get_connects(nodea=current_node):
            logger.debug("# iterate over get_connects() {0}".format(connect.tmpl_connect.name))
            if connect.tail_node == current_node or connect.head_node == current_node:
                logger.debug("#has current_node")
                ikeid = connect.id
                ca = connect.tmpl_connect.cred_auth

                #The start_action field (0 = ACTION_NONE/ADD, 1 = ACTION_ROUTE, 2 = ACTION_START/RESTART)
                #in the child_configs table. Always set to 2
                start_action = 2

                #The dpd_action field (0 = CLEAR, 1 = ???, 2 = START/RESTART)
                #2 for amon and 0 for sphynx except if two amon, à on one side and 2 on the other side
                if current_node.tmpl_node.allow_master:
                    #start_action = 2
                    dpd_action = 2
                else:
                    #start_action = 0
                    dpd_action = 0

                #search local node
                #search remote node
                if connect.tail_node == current_node:
                    nodea = connect.tail_node
                    creda = connect.tail_cred
                    nodeb = connect.head_node
                    credb = connect.head_cred
                    local = connect.tail_extr.priv_ip
                    remote = connect.head_extr.pub_ip
                    if nodeb.tmpl_node.mimetype == current_node.tmpl_node.mimetype:
                        #start_action = 0
                        dpd_action = 0
                    name_peercfg = '%s-%s' % (nodea.name, nodeb.name)
                else:
                    nodea = connect.head_node
                    creda = connect.head_cred
                    nodeb = connect.tail_node
                    credb = connect.tail_cred
                    local = connect.head_extr.priv_ip
                    remote = connect.tail_extr.pub_ip
                    if nodeb.tmpl_node.mimetype == current_node.tmpl_node.mimetype:
                        #start_action = 2
                        dpd_action = 2
                    name_peercfg = '%s-%s' % (nodeb.name, nodea.name)

                if creda == None:
                    logger.debug("No credential for %s in connect %s"%(nodea.name, connect.tmpl_connect.name))
                    continue
                if credb == None:
                    logger.debug("No credential for %s in connect %s"%(nodeb.name, connect.tmpl_connect.name))
                    continue
                #ret = add_certificates_and_identities(nodea, nodeb)
                ret = add_certificates_and_identities(nodea, creda, nodeb, credb)

                sw_create_ike_configs(ikeid, local, remote)

                local_id = _make_cred_id(creda, '9')
                remote_id = _make_cred_id(credb, '9')
                #peer_id = local_id
                sw_create_peer_configs(name=name_peercfg, id=ikeid, ike_cfg=ikeid,
                                       local_id=local_id, remote_id=remote_id, virtual=None, pool=None)
                #get all edges for this connect
                logger.debug('edges : {0}'.format(str(connect.edges)))
                for edge in connect.edges:
                    name_childcfg = edge.tmpl_edge.name
                    logger.debug("# iterate over edges {0}".format(name_childcfg))
                    edgeid = edge.id

                    if current_node.tmpl_node.mimetype == u'sphynx':
                        updown = ''
                    else:
                        updown = '/etc/ipsec.d/ipsec_updown'
                    sw_create_child_config(childid=edgeid, name=name_childcfg, updown=updown, dpd_action=dpd_action, start_action=start_action)
                    sw_create_child_config_proposal(child_cfg=edgeid, prio=1, prop=3)
                    sw_join_child_peer(peer_id=ikeid, childid=edgeid)

                    tmplvertex_a = edge.tmpl_edge.tmpl_vertex_a
                    tmplvertex_b = edge.tmpl_edge.tmpl_vertex_b

                    if nodea.tmpl_node == tmplvertex_a.tmpl_node:
                        if  nodeb.tmpl_node != tmplvertex_b.tmpl_node:
                            logger.error('no relation between nodeb et tmplvertex_b')
                            raise Exception('no relation between nodeb et tmplvertex_b')
                        vertex_a = get_vertex_by_node_tmplvertex(node=nodea, tmpl_vertex=tmplvertex_a)
                        vertex_b = get_vertex_by_node_tmplvertex(node=nodeb, tmpl_vertex=tmplvertex_b)
                        tmplvertex_name_a = tmplvertex_a.name
                        tmplvertex_name_b = tmplvertex_b.name
                    elif nodea.tmpl_node == tmplvertex_b.tmpl_node:
                        if  nodeb.tmpl_node != tmplvertex_a.tmpl_node:
                            logger.error('no relation between nodeb et tmplvertex_a')
                            raise Exception('no relation between nodeb et tmplvertex_a')
                        vertex_a = get_vertex_by_node_tmplvertex(node=nodea, tmpl_vertex=tmplvertex_b)
                        vertex_b = get_vertex_by_node_tmplvertex(node=nodeb, tmpl_vertex=tmplvertex_a)
                        tmplvertex_name_a = tmplvertex_b.name
                        tmplvertex_name_b = tmplvertex_a.name
                    else:
                        logger.error('no relation between nodea et tmplvertex')
                        raise Exception('no relation between nodea et tmplvertex')

                    if vertex_a == None:
                        logger.debug("No vertex for node %s and tmpl_vertex %s"%(nodea.name, tmplvertex_name_a))
                        continue
                    if vertex_b == None:
                        logger.debug("No vertex for node %s and tmpl_vertex %s"%(nodeb.name, tmplvertex_name_b))
                        continue

                    ##
                    create_traffic_selector(vertex_a)
                    create_traffic_selector(vertex_b)

                    sw_join_config_traffic(childid=edgeid, trafficid=vertex_a.id, kind=0)
                    sw_join_config_traffic(childid=edgeid, trafficid=vertex_b.id, kind=1)
            else:
                logger.debug("no %s node for %s" % (current_node.name, connect.tmpl_connect.name))
    tmpl_nodes = get_tmpl_nodes(u'sphynx')

    if len(tmpl_nodes) ==0:
        raise Exception('No available connect')
    if tmpl_nodes == []:
        raise Exception("No template for sphynx")
    if len(tmpl_nodes) != 1:
        raise Exception('More than one tmpl sphynx in database !')
    if tmpl_nodes[0].nodes == []:
        raise Exception("No sphynx in template")
    if len(tmpl_nodes[0].nodes) != 1:
        raise Exception('More than one sphynx in database !')
    current_node = tmpl_nodes[0].nodes[0]
    if isfile(strongswan_database):
        unlink(strongswan_database)

    create_database(current_node, 'sqlite:///%s'%strongswan_database)

    if not isdir(strongswan_databases_directory):
        raise Exception('Directory %s not found' % strongswan_databases_directory)
    for node in get_nodes():
        if node == current_node:
            continue
        database = '%s%s.db'% (strongswan_databases_directory, node.name)
        if isfile(database):
            unlink(database)
        create_database(node, 'sqlite:///%s'%database)
        uai = node.uai
        name = node.name
        if uai != None:
            logger.debug('sending database : %s and uai : %s to zephir' % (database, uai))
            vpnpath = "/home/data/vpn"
            # database archive
            if not isdir(dirname(vpnpath)):
                mkdir(dirname(vpnpath))
            if not isdir(vpnpath):
                mkdir(vpnpath)
            amonpath = vpnpath + sep + str(uai)
            archivename = str(uai) + "-" + str(name) + ".tar.gz"
            if isdir(amonpath):
                rmtree(amonpath)
            mkdir(amonpath)
            copy(database, amonpath + sep + "ipsec.db")
            unlink(database)
            name_test_rvp = amonpath + sep + "test-rvp"
            ftest_rvp = open (name_test_rvp,"w")
            ftest_rvp.write(test_rvp)
            ftest_rvp.close()
            system("chmod 755 {0}".format(name_test_rvp))
            system("""cd "{0}" ; tar -czf "{2}" {1}""".format(amonpath, "ipsec.db test-rvp", archivename))
            archive = amonpath + sep + archivename
            remove(amonpath + sep + "ipsec.db")
            remove(name_test_rvp)
            if zephir != None:
                zephir.send_db(archive, uai, name)

