# coding: utf-8
import xmlrpclib
try:
    from zephir.zephir_conf.zephir_conf import id_serveur, adresse_zephir
    ZEPHIR = True
except:
    ZEPHIR = False
from arv.lib.util import trace
from arv.lib.logger import logger

ZEPHIR_PERM_CODE = 7
ZEPHIR_MODULE_VERSION = 5

class Zephir:
    @trace
    def __init__(self, user, password):
        try:
            self.zephir = xmlrpclib.ServerProxy('https://%s:%s@%s:7080'%(user, password, adresse_zephir))
            #test connection
            perm = self.zephir.get_permissions(user)
            if perm[0] != 1 or ZEPHIR_PERM_CODE not in perm[1]:
                raise Exception('Unable to connect to Zéphir')
        except:
            raise Exception('Unable to connect to Zéphir')

    @trace
    def get_amons(self):
        try:
            ret = []
            resp_code, modules = self.zephir.modules.get_module()
            mod_amon = []
            if resp_code == 1:
                for module in modules:
                    if module['libelle'] in ['amon-2.3', 'amonecole-2.3']:
                        mod_amon.append(module['id'])

                if mod_amon == []:
                    raise Exception("Unable to get module 'amon-2.3' from Zéphir")
            else:
                raise Exception('Unable to get modules from Zéphir')
            servers = []
            for mod in mod_amon:
                servers.extend(self.get_etab_module_servers(mod))
            if servers != []:
                for amon in servers:
                    logger.debug("#-> amon {0} - {1}".format(unicode(amon['libelle']), unicode(amon['rne'])))
                    ret.append((unicode(amon['libelle']), unicode(amon['rne'])))

            #etabs = self.zephir.etabs.get_etab()
            #if etabs[0] != 1:
            #    raise Exception('Zéphir should reply 1')
            #logger.debug('etabs: %s' % str(etabs))
            #for etab in etabs[1]:
            #    uai = etab['rne']
            #    module = self.get_module('amon')
            #    logger.debug("#-> get_module amon retrieved : " + str(module))
            #    if module != None:
            #        module_id = int(module['moduleid'])
            #        servers = self.get_etab_module_servers(uai, module_id)
            #        if servers != None:
            #            for amon in servers:
            #                logger.debug("#-> amon {0} - {1}".format(unicode(amon['libelle']), unicode(etab['rne'])))
            #                ret.append((unicode(amon['libelle']), unicode(etab['rne'])))
            return ret
        except:
            raise Exception('Error: unable to get data from Zéphir')

    @trace
    def get_sphynxs(self):
        try:
            ret = []
            resp_code, modules = self.zephir.modules.get_module()
            mod_sphynx = None
            if resp_code == 1:
                for module in modules:
                    if module['libelle'] == 'sphynx-2.3':
                        mod_sphynx = module['id']
                        break
                if mod_sphynx is None:
                    raise Exception("Unable to get module 'sphynx-2.3' from Zéphir")
            else:
                raise Exception('Unable to get modules from Zéphir')
            servers = self.get_etab_module_servers(mod_sphynx)
            if servers != []:
                for sphynx in servers:
                    logger.debug("#-> sphynx {0} - {1}".format(unicode(sphynx['libelle']), unicode(sphynx['rne'])))
                    ret.append((unicode(sphynx['libelle']), unicode(sphynx['rne'])))
#            ret = []
#            etabs = self.zephir.etabs.get_etab()
#            if etabs[0] != 1:
#                raise Exception('Zéphir should reply 1')
#            logger.debug('etabs: %s' % str(etabs))
#            for etab in etabs[1]:
#                uai = etab['rne']
#                module = self.get_module('sphynx')
#                logger.debug("#-> get_module amon retrieved : " + str(module))
#                if module != None:
#                    module_id = int(module['moduleid'])
#                    servers = self.get_etab_module_servers(uai, module_id)
#                    if servers != None:
#                        for amon in servers:
#                            logger.debug("#-> amon {0} - {1}".format(unicode(amon['libelle']), unicode(etab['rne'])))
#                            ret.append((unicode(amon['libelle']), unicode(etab['rne'])))
            return ret
        except:
            raise Exception('Error: unable to get data from Zéphir')

    @trace
    def get_modules(self):
        try:
            ret = []
            tmodules = self.zephir.modules.get_module()
            if tmodules[0] != 1:
                raise exception('error: unable to get data from zéphir')
            logger.debug('zephir get_module : %s' % str(tmodules[1]))
            for module in tmodules[1]:
                if module['version'] == ZEPHIR_MODULE_VERSION:
                    name = module['libelle'].split('-')[0]
                    ret.append({'name': name, 'moduleid': module['id']})
            return ret
        except:
            raise Exception('Error: unable to get data from Zéphir')
    @trace
    def get_module(self, module):
        try:
            #module = '{0}-2.3'.format(module)
            server = None
            tmodules = self.get_modules()
            logger.debug('got the tmodules: ' +str(tmodules))
            for tmodule in tmodules:
                if tmodule['name'] == module:
                    return tmodule
            return None
        except:
            raise Exception('Error: unable to get data from Zéphir')

    @trace
    def get_etab_module(self, uai, name, module_id):
        try:
            tserver = self.zephir.serveurs.groupe_serveur({'rne': uai, 'libelle':name, 'module_actuel': module_id})
            if tserver[0] != 1:
                raise Exception('Error: unable to get data from Zéphir')
            if tserver[1] == []:
                return None
            return tserver[1][0]
        except:
            raise Exception('Error: unable to get data from Zéphir')

    @trace
    def get_etab_module_servers(self, module_id):
        try:
            tserver = self.zephir.serveurs.groupe_serveur({'module_actuel': module_id})
            if tserver[0] != 1:
                raise Exception('Error: unable to get data from Zéphir')
            return tserver[1]
        except:
            raise Exception('Error: unable to get data from Zéphir')

    @trace
    def get_var(self, uai, name, module, var):
        try:
            server = self.get_etab_module(uai, name, module)
            if server != None:
                sid = server['id']
                vars = self.zephir.serveurs.get_config(sid)

                if vars[0] != 1:
                    raise Exception('Error: unable to get data from Zéphir')

                return vars[1][var]
            return None
        except:
            #FIXME
            import traceback
            traceback.print_exc()
            raise Exception('Error: unable to get data from Zéphir')

    @trace
    def get_variables(self, module):
        try:
            ret=[]
            variables = self.zephir.modules.get_vars(module)
            if variables[0] != 1:
                raise exception('error: unable to get variables from Zéphir for %s' % module)
            logger.debug('zephir variables for %s: %s' % (module, str(variables)))
            for variable in variables[1].keys():
                ret.append({'name': variable})
            return ret
        except:
            import traceback
            traceback.print_exc()
            raise Exception('Error: unable to get variable from Zéphir')

    @trace
    def send_db(self, archive, uai, name):
        try:
            f = open(str(archive), 'rb')
            str_obj = xmlrpclib.base64.encodestring(f.read())
            f.close()
            module = self.get_module('amon')
            logger.debug("#-> get_module amon retrieved : " + str(module))
            if module != None:
                module_id = int(module['moduleid'])
                logger.debug("#-> module id " + str(module_id))
                server = self.get_etab_module(uai, name, module_id)
                logger.debug("#-> server " + str(server))
                if server != None:
                    server_id = int(server['id'])
                    logger.debug("#-> server id {0}".format(server_id))
                    res = self.zephir.uucp.sphynx_add(id_serveur, server_id, str_obj)
                    if res[0] != 1:
                        raise Exception("unable to send data to zephir : %s" % str(res[1]))
                else:
                    #FIXME: pas de print
                    print "pas d'Amon pour %s"%uai
            else:
                raise Exception("No Amon's module in Zéphir")
        except:
            import traceback
            traceback.print_exc()
            raise Exception('Error: unable to send data to zephir')
