# -*- coding: UTF-8 -*-
"""website add pages
"""
from arv.web.page import Page, usezephir, IPreferences
from arv.db.edge import *
from arv.db.node import *
from arv.lib.factory import get_object_by_id
from arv.lib.util import trace, get_req_archive
from arv.lib.logger import logger

# ____________________________________________________________
# save pages root
class SavePage(Page):
    """save pages root page
    """
    save_database = True

# ____________________________________________________________
# add pages
class AddTmplConnect(SavePage):
    def render_page(self, request):
        name = self.get_data_from_request(request=request, arg='name')
        tmpl_node1 = self.get_data_from_request(request=request, arg='tmplnode1', object_type='tmpl_node')
        tmpl_node2 = self.get_data_from_request(request=request, arg='tmplnode2', object_type='tmpl_node')
        credauth = self.get_data_from_request(request=request, arg='credauth', object_type='credential')
        if not credauth.ca:
            raise Exception('Credential need to be authority')
        add_tmpl_connect(name, tmpl_node1, tmpl_node2, credauth)
        return "TmplConnect added"

class AddTmplNode(SavePage):
    def render_page(self, request):
        name = request.args['name'][0]
        add_tmpl_node(name)
        return 'TmplNode added'

class AddTmplEdge(SavePage):
    def render_page(self, request):
        name = self.get_data_from_request(request=request, arg='name')
        tmpl_vertex1 = self.get_data_from_request(request=request, arg='tmplvertex1', object_type='tmpl_vertex')
        tmpl_vertex2 = self.get_data_from_request(request=request, arg='tmplvertex2', object_type='tmpl_vertex')
        tmpl_connect = self.get_data_from_request(request=request, arg='id', object_type='tmpl_connect')

        tmpl_connect.add_tmpl_edge(name, tmpl_vertex1, tmpl_vertex2)
        return 'TmplEdge added'

class AddTmplVertex(SavePage):
    def render_page(self, request):
        name = self.get_data_from_request(request=request, arg='name')
        tmpl_node = self.get_data_from_request(request=request, arg='id', object_type='tmpl_node')
        mimetype = self.get_data_from_request(request=request, arg='mimetype')
        zephir_module = self.get_data_from_request(request=request, arg='moduleid')
        zephir_var_ip1 = self.get_data_from_request(request=request, arg='ip1')
        zephir_var_ip2 = self.get_data_from_request(request=request, arg='ip2', required = False)

        tmpl_vertex1 = tmpl_node.add_tmpl_vertex(name=name,
                mimetype=mimetype,
                zephir_module=zephir_module,
                zephir_var_ip1=zephir_var_ip1,
                zephir_var_ip2=zephir_var_ip2)
        return 'TmplVertex added'


class AddNode(SavePage):
    @trace
    def render_page(self, request):
        name =  self.get_data_from_request(request=request, arg='name')
        uai =  self.get_data_from_request(request=request, arg='uai')
        tmpl_node = self.get_data_from_request(request=request, arg='id', object_type='tmpl_node', required=False)
        logger.debug('#-> add_node param :name: ' + name)
        if tmpl_node == None:
            tmpl_node = get_tmpl_nodes(mimetype=u'etablissement')[0]
        vertices = self.get_data_from_request(request=request, arg='vertices', load_json=True)

        node = tmpl_node.add_node(name=name, uai=uai)
        for tvid, ip in vertices.items():
            if ip != ["", ""]:
                tmpl_vertex = get_object_by_id(int(tvid), 'tmpl_vertex')
                ip1 = ip[0]
                ip2 = ip[1]
                if ip2 == "":
                    ip2 = None
                tmpl_vertex.add_vertex(ip1=ip1, ip2=ip2, node=node)
        return 'Node added'

class AddConnect(SavePage):
    def render_page(self, request):
        tail_node = self.get_data_from_request(request=request, arg='node1', object_type='node')
        head_node = self.get_data_from_request(request=request, arg='id', object_type='node')

        tail_extr = self.get_data_from_request(request=request, arg='extr1', object_type='extremity')
        head_extr = self.get_data_from_request(request=request, arg='extr2', object_type='extremity')

        tail_cred = self.get_data_from_request(request=request, arg='cred1', object_type='credential')
        head_cred = self.get_data_from_request(request=request, arg='cred2', object_type='credential')

        tmpl_connect = self.get_data_from_request(request=request, arg='tmpl_connect', object_type='tmpl_connect')
        edges = self.get_data_from_request(request=request, arg='edges', load_json=True)

        connect = tmpl_connect.add_connect(tail_node, head_node,
                tail_extr, head_extr, tail_cred, head_cred)
        for tmpledgeid, activated in edges.items():
            if activated:
                tmpl_edge = get_object_by_id(tmpledgeid, 'tmpl_edge')
                tmpl_edge.add_edge(connect)
        return 'Connect added'

class AddImportedCred(SavePage):
    def render_page(self, request):
        private_key = self.get_data_from_request(request=request, arg='private_key')
        public_key = self.get_data_from_request(request=request, arg='public_key')
        password = self.get_data_from_request(request=request, arg='password', required=True)
        node = self.get_data_from_request(request=request, arg='node', object_type='node')
        node.import_credential(private_key=private_key, credential=public_key, passwd=password)
        return 'Credential imported'

class AddGeneratedCred(SavePage):
    def render_page(self, request):
        name = self.get_data_from_request(request=request, arg='name')
        passwd = self.get_data_from_request(request=request, arg='passwd')
        node = self.get_data_from_request(request=request, arg='node', object_type='node')
        #cred_priv_type = self.get_data_from_request(request=request, arg='cred_priv_type')
        cred_priv_type = request.args['cred_priv_type'][0]
        logger.debug("#-> cred_priv_type" + cred_priv_type)
        credential = add_credential(name, passwd, node, cred_priv_type)
        if credential != True:
            return ("download_name", 'arv/dl_credential?name=%s'%name, 'Credential added')
        return 'Credential added'

class DlCredential(SavePage):
    def render_page(self, request):
        #return ("download_file", 'toto.cert', "ca c'est du gros fichier.")
        name = self.get_data_from_request(request=request, arg='name')
        #logger.debug('unsigned_credential in DlCredential: %s'%str(credential.unsigned_credential))
        reqfile_content = get_req_archive(name)
        return ("download_file", name+".tgz", reqfile_content) #credential.unsigned_credential)

class AddCredAuth(SavePage):
    def render_page(self, request):
        credentialauth = self.get_data_from_request(request=request, arg='credentialauth')
        passwd = self.get_data_from_request(request=request, arg='passwd')
        add_credential_auth(credential=credentialauth, passwd=passwd)
        return 'Credential auth added'

class AddExtremity(SavePage):
    def render_page(self, request):
        pub_ip = request.args['pub_ip'][0]
        priv_ip = request.args['priv_ip'][0]
        if priv_ip == "":
            priv_ip = None
        node = self.get_data_from_request(request=request, arg='id', object_type='node')
        node.add_extremity(pub_ip=pub_ip, priv_ip=priv_ip)
        return 'Extremity added'

# ____________________________________________________________
# mod pages

class ModNode(SavePage):
    @trace
    def render_page(self, request):
        node = self.get_data_from_request(request=request, arg='node',  object_type='node')
        vertices = self.get_data_from_request(request=request, arg='vertices', load_json=True)
        node_vertices = {}
        for vertex in node.vertices:
            if vertex.tmpl_vertex:
                node_vertices[vertex.tmpl_vertex.id] = vertex
        for tvid, ip in vertices.items():
            if ip != ["", ""]:
                ip1 = ip[0]
                ip2 = ip[1]
                if ip2 == "":
                    ip2 = None
                tvid = int(tvid)
                if not tvid in node_vertices:
                    tmpl_vertex = get_object_by_id(tvid, 'tmpl_vertex')
                    tmpl_vertex.add_vertex(ip1=ip1, ip2=ip2, node=node)
                else:
                    node_vertices[tvid].ip1 = ip1
                    node_vertices[tvid].ip1 = ip1
        return 'Node modified'

class ModTmplEdge(SavePage):
    def render_page(self, request):
        tmpl_edge = self.get_data_from_request(request=request, arg='id', object_type='tmpl_edge')
        name = self.get_data_from_request(request=request, arg='name')
        tmpl_vertex1 = self.get_data_from_request(request=request, arg='tmplvertex1', object_type='tmpl_vertex')
        tmpl_vertex2 = self.get_data_from_request(request=request, arg='tmplvertex2', object_type='tmpl_vertex')

        tmpl_edge.mod_name(name)
        tmpl_edge.mod_tmpl_vertex1(tmpl_vertex1)
        tmpl_edge.mod_tmpl_vertex2(tmpl_vertex2)
        return 'TmplEdge modified'

class ModTmplVertex(SavePage):
    def render_page(self, request):
        tmpl_vertex = self.get_data_from_request(request=request, arg='tmpl_vertex_id', object_type='tmpl_vertex')
        name = self.get_data_from_request(request=request, arg='name')
        tmpl_node = self.get_data_from_request(request=request, arg='id', object_type='tmpl_node')
        mimetype = self.get_data_from_request(request=request, arg='mimetype')
        zephir_module = self.get_data_from_request(request=request, arg='moduleid')
        zephir_var_ip1 = self.get_data_from_request(request=request, arg='ip1')
        zephir_var_ip2 = self.get_data_from_request(request=request, arg='ip2', required = False)

        tmpl_vertex.mod_name(name)
        tmpl_vertex.mod_mimetype(mimetype)
        tmpl_vertex.mod_zephir(zephir_module, zephir_var_ip1, zephir_var_ip2)

        return 'TmplVertex modified'

class ModConnect(SavePage):
    def render_page(self, request):
        connect = self.get_data_from_request(request=request, arg='connect_id', object_type='connect')
        tail_extr = self.get_data_from_request(request=request, arg='extr1', object_type='extremity')
        head_extr = self.get_data_from_request(request=request, arg='extr2', object_type='extremity')
        tail_cred = self.get_data_from_request(request=request, arg='cred1', object_type='credential')
        head_cred = self.get_data_from_request(request=request, arg='cred2', object_type='credential')
        edges = self.get_data_from_request(request=request, arg='edges', load_json=True)

        connect.mod_tail_extr(tail_extr)
        connect.mod_head_extr(head_extr)

        connect.mod_tail_cred(tail_cred)
        connect.mod_head_cred(head_cred)

        current_edges = {}
        for edge in connect.edges:
            current_edges[edge.tmpl_edge.id] = edge
        current_tmpledges_id = current_edges.keys()
        for tmpledgeid, activated in edges.items():
            if activated:
                #if activated and not in current connect, add it
                if tmpledgeid not in current_tmpledges_id:
                    tmpl_edge = get_object_by_id(tmpledgeid, 'tmpl_edge')
                    tmpl_edge.add_edge(connect)
            else:
                #if not activated and in current connect, remove it
                if tmpledgeid in current_tmpledges_id:
                    connect.edges.pop(current_edges[tmpledgeid])
        #remove edges in current connect not in new connect
        for extra_id in list(set(current_tmpledges_id)-set(edges.keys())):
            connect.edges.remove(current_edges[extra_id])

        return 'Connect modified'

# ____________________________________________________________
# del pages

class DelTmplConnect(SavePage):
    def render_page(self, request):
        tmpl_connect = self.get_data_from_request(request=request, arg='deleteid', object_type='tmpl_connect')
        del_tmpl_connect(tmpl_connect)

        return ''

class DelTmplNode(SavePage):
    def render_page(self, request):
        tmpl_node = self.get_data_from_request(request=request, arg='deleteid', object_type='tmpl_node')
        del_tmpl_node(tmpl_node)

        return ''

class DelTmplEdge(SavePage):
    def render_page(self, request):
        tmpl_edge = self.get_data_from_request(request=request, arg='deleteid', object_type='tmpl_edge')
        del_tmpl_edge(tmpl_edge)

        return ''

class DelNode(SavePage):
    def render_page(self, request):
        node = self.get_data_from_request(request=request, arg='deleteid', object_type='node')
        del_node(node)
        return 'Node deleted'

class DelEdge(SavePage):
    def render_page(self, request):
        obj = self.get_data_from_request(request=request, arg='deleteid', object_type='edge')
        connect = obj.connect
        del_conn = False
        if connect.edges == [obj]:
            del_conn = True
        del_edge(obj)
        if del_conn:
            del_connect(connect)
            return 'Edge and connect deleted'
        return 'Edge deleted'

class DelConnect(SavePage):
    def render_page(self, request):
        connect = self.get_data_from_request(request=request, arg='deleteid', object_type='connect')
        for edge in connect.edges:
            del_edge(edge)
        del_connect(connect)
        return 'Connect deleted'

class DelCred(SavePage):
    def render_page(self, request):
        obj = self.get_data_from_request(request=request, arg='deleteid', object_type='credential')
        obj.delete()
        return 'Credential deleted'

# ____________________________________________________________
# get pages
class GetNodes(Page):
    def render_page(self, request):
        ret = []
        connected = self.get_data_from_request(request=request, arg='connected', required=False)
        if connected == 'true':
            nodea = self.get_data_from_request(request=request, arg='id', object_type='node')
            nodes = get_connected_nodes(nodea)
        else:
            nodea = self.get_data_from_request(request=request, arg='id', object_type='node', required=False)
            nodes = get_nodes(nodea)
        for node in nodes:
            ret.append({'node': node.name, 'id': node.id})
        return ret

class GetNodesState(Page):
    def render_page(self, request):
        ret = []
        nodes = get_nodes()
        #FIXME: n'a plus vraiment de sens ...
        for node in nodes:
            connects = []
            #we extend to copy node's ref in connects not extend node.tail_conns
            connects.extend(node.tail_conns)
            connects.extend(node.head_conns)
            creds_auth = []
            state = True
            for connect in connects:
                cred_auth = connect.tmpl_connect.cred_auth
                if get_credential_by_ca_node(cred_auth, node) == None:
                    state = False
                    break
            ret.append({'node': node.name, 'id': node.id, 'state': state})
        return ret

class GetEdges(Page):
    def render_page(self, request):
        ret = []
        nodea = self.get_data_from_request(request=request, arg='nodea', object_type='node')
        nodeb = self.get_data_from_request(request=request, arg='nodeb', object_type='node')
        for connect in get_connects(nodea, nodeb):
            if connect.edges == []:
                ret.append({'connect': connect.tmpl_connect.name})
            else:
                for edge in connect.edges:
                    if edge.tmpl_edge.tmpl_vertex_a.tmpl_node == nodea.tmpl_node:
                        vertexa = edge.tmpl_edge.tmpl_vertex_a.name
                        vertexb = edge.tmpl_edge.tmpl_vertex_b.name
                    else:
                        vertexa = edge.tmpl_edge.tmpl_vertex_b.name
                        vertexb = edge.tmpl_edge.tmpl_vertex_a.name
                    ret.append({'edge': edge.tmpl_edge.name, 'id': edge.id,
                            'connect': connect.tmpl_connect.name,
                            'vertexa': vertexa , 'vertexb': vertexb,
                            'activate': False})
        return ret

class GetConnects(Page):
    def render_page(self, request):
        ret = []
        nodea = self.get_data_from_request(request=request, arg='nodea', object_type='node')
        nodeb = self.get_data_from_request(request=request, arg='nodeb', object_type='node')
        for connect in get_connects(nodea, nodeb):
            ret.append({'id': connect.id, 'connect': connect.tmpl_connect.name})
        return ret

class GetTmplNodes(Page):
    def render_page(self, request):
        ret = []
        exclude = self.get_data_from_request(request=request, arg='exclude', object_type='tmpl_node', required=False)
        mimetype = self.get_data_from_request(request=request, arg='mimetype', required=False)
        for tmplnode in get_tmpl_nodes(mimetype=mimetype):
            if tmplnode != exclude:
                ret.append({'tmplnode': tmplnode.name, 'id': tmplnode.id})
        return ret

class GetTmplConnects(Page):
    def render_page(self, request):
        ret = []
        node_a = self.get_data_from_request(request=request, arg='node_a_id', object_type='node', required=False)
        if node_a != None:
            node_b = self.get_data_from_request(request=request, arg='node_b_id', object_type='node')
            tmpl_connects = []
            for connect in get_connects(node_a, node_b):
                tmpl_connects.append(connect.tmpl_connect)
            tmpl_node_a = node_a.tmpl_node
            tmpl_node_b = node_b.tmpl_node
        else:
            tmpl_node_a= None
            tmpl_node_b = None
            tmpl_connects = []
        for tmpl_connect in get_tmpl_connects(tmpl_node_a, tmpl_node_b):
            if tmpl_connect not in tmpl_connects:
                ret.append({'tmplconnect': tmpl_connect.name,
                        'id': tmpl_connect.id,
                        'tmplnodea': tmpl_connect.tmpl_node_a.name,
                        'tmplnodeaid': tmpl_connect.tmpl_node_a.id,
                        'tmplnodeb': tmpl_connect.tmpl_node_b.name,
                        'tmplnodebid': tmpl_connect.tmpl_node_b.id})
        return ret

class GetTmplEdges(Page):
    def render_page(self, request):
        ret = []
        tmpl_connect = self.get_data_from_request(request=request, arg='id', object_type='tmpl_connect')
        connect = self.get_data_from_request(request=request, arg='connect_id', object_type='connect', required=False)
        tmpl_edges = []
        if connect != None:
            for edge in connect.edges:
               tmpl_edges.append(edge.tmpl_edge)
        for tmpledge in get_tmpl_edges(tmpl_connect):
            rettedge = {'id': tmpledge.id,
                'tmplvertexname': tmpledge.name,
                'tmplvertex1': tmpledge.tmpl_vertex_a.name,
                'tmplvertex1id': tmpledge.tmpl_vertex_a.id,
                'tmplvertex2': tmpledge.tmpl_vertex_b.name,
                'tmplvertex2id': tmpledge.tmpl_vertex_b.id,
            }
            if tmpledge in tmpl_edges:
                rettedge['activate'] = True
            ret.append(rettedge)
        return ret

class GetTmplVertices(Page):
    @trace
    def render_page(self, request):
        ret = []
        tmpl_node = self.get_data_from_request(request=request, arg='tmplnodeid', object_type='tmpl_node', required=False)
        node = self.get_data_from_request(request=request, arg='nodeid', object_type='node', required=False)
        uai = self.get_data_from_request(request=request, arg='uai', required=False)
        name = self.get_data_from_request(request=request, arg='name', required=False)
        if None not in [tmpl_node, node] or tmpl_node == node:
            raise Exception('Must specified tmpl_node or node')
        if node != None:
            tmpl_node = node.tmpl_node
            tmpl_vertex = {}
            for vertex in node.vertices:
                tmpl_vertex[vertex.tmpl_vertex] = (vertex.ip1, vertex.ip2)

        for tmplvertex in get_tmpl_vertices(tmpl_node):
            ip1 = ''
            ip2 = ''
            module = ''
            if node != None:
                ip1, ip2 = tmpl_vertex.get(tmplvertex, ('', ''))
            else:
                if uai != None and usezephir == True:
                    prefs = request.getSession(IPreferences)
                    if prefs.zephir != None:
                        if tmplvertex.zephir_module:
                            ip1 = prefs.zephir.get_var(uai, name, tmplvertex.zephir_module, tmplvertex.zephir_var_ip1)
                            if tmplvertex.mimetype != u'ip':
                                ip2 = prefs.zephir.get_var(uai, name, tmplvertex.zephir_module, tmplvertex.zephir_var_ip2)
                else:
                    #if search tmplVertex attribute
                    module = tmplvertex.zephir_module
                    ip1 = tmplvertex.zephir_var_ip1
                    ip2 = tmplvertex.zephir_var_ip2
            ret.append({'tmplvertex': tmplvertex.name,
                'id': tmplvertex.id,
                'mimetype': tmplvertex.mimetype,
                'module': module,
                'ip1': ip1,
                'ip2': ip2
            })

        return ret

class GetCred(Page):
    def render_page(self, request):
        ret = []
        node = self.get_data_from_request(request=request, arg='node', object_type='node')
        connect = self.get_data_from_request(request=request, arg='connect_id', object_type='connect', required=False)
        connect_cred = None
        if connect != None:
            if node == connect.tail_node:
                connect_cred = connect.tail_cred.id
            elif node == connect.head_node:
                connect_cred = connect.head_cred.id
            else:
                raise Exception('Node is not in connect')
        for cred in node.get_credentials():
            retcred = {'id':cred.id, 'cred':cred.name}
            if retcred['id'] == connect_cred:
                retcred['selected'] = True
            ret.append(retcred)
        return ret

class GetCredAuth(Page):
    def render_page(self, request):
        ret = []
        for credauth in get_credentials_auth():
            ret.append({'id':credauth.id, 'credauth':credauth.name})
        return ret

class GetExtr(Page):
    def render_page(self, request):
        ret = []
        node = self.get_data_from_request(request=request, arg='node', object_type='node')
        connect = self.get_data_from_request(request=request, arg='connect_id', object_type='connect', required=False)
        connect_extr = None
        if connect != None:
            if node == connect.tail_node:
                connect_extr = connect.tail_extr.id
            elif node == connect.head_node:
                connect_extr = connect.head_extr.id
            else:
                raise Exception('Node is not in connect')
        for extr in node.get_extremities():
            retextr = {'id': extr.id, 'extr': extr.pub_ip}
            if connect_extr != None:
                if retextr['id'] == connect_extr:
                    retextr['selected'] = True
            ret.append(retextr)
        return ret

class GetZephirEtabs(Page):
    @trace
    def render_page(self, request):
        ret = []
        if usezephir == True:
            prefs = request.getSession(IPreferences)
            if prefs.zephir != None:
                zetab = set(prefs.zephir.get_amons())
                letab = set(get_etabs())
                #return only zetab not in letab
                keyword = ('libelle', 'uai')
                for etab in list(zetab - letab):
                    ret.append(dict(zip(keyword, etab)))
        return ret

class GetZephirModules(Page):
    def render_page(self, request):
        ret = []
        if usezephir == True:
            prefs = request.getSession(IPreferences)
            if prefs.zephir != None:
                 ret = prefs.zephir.get_modules()
        #Insert empty field
        ret.insert(0, {'name': None, 'id': None})
        return ret

class GetZephirVariables(Page):
    def render_page(self, request):
        module = self.get_data_from_request(request=request, arg='module')
        if module == '':
            return []
        ret = []
        if usezephir == True:
            prefs = request.getSession(IPreferences)
            if prefs.zephir != None:
                ret = prefs.zephir.get_variables(module)
        return ret

